{
TGMPolygon component

  ES: componente para mostrar polgonos en un mapa de Google Maps mediante el
      componente TGMMap
  EN: component to show polygons on Google Map map using the component TGMMap

=========================================================================
MODO DE USO/HOW TO USE

  ES: poner el componente en el formulario, linkarlo a un TGMMap y poner los
      polgonos a mostrar
  EN: put the component into a form, link to a TGMMap and put the polygons to show
=========================================================================
History:

ver 0.1.3
  ES:
    cambio: mtodos Set cambiados para evitar duplicidad de cdigo
  EN:
    change: changed Set methods to avoid duplicate code

ver 0.1.2:
  ES- primera versin
  EN- first version
=========================================================================
IMPORTANTE PROGRAMADORES: Por favor, si tienes comentarios, mejoras,
  ampliaciones, errores y/o cualquier otro tipo de sugerencia, envame un correo a:
  gmlib@cadetill.com

IMPORTANT PROGRAMMERS: please, if you have comments, improvements, enlargements,
  errors and/or any another type of suggestion, please send me a mail to:
  gmlib@cadetill.com
=========================================================================

Copyright () 2012, by Xavier Martinez (cadetill)

@author Xavier Martinez (cadetill)
@web  http://www.cadetill.com
}
unit GMPolygon;

interface

uses
  Classes, Graphics,
  GMPolyline, GMMap;

type
  { ****************************************************************************
    internal class
  **************************************************************************** }
  TPolygon = class(TPolyline)
  private
    FFillColor: TColor;
    FFillOpacity: Real;
    procedure SetFillColor(const Value: TColor);
    procedure SetFillOpacity(const Value: Real);
  protected
  public
    constructor Create(Collection: TCollection); override;

    procedure Assign(Source: TPersistent); override;
  published
    property FillColor: TColor read FFillColor write SetFillColor;
    property FillOpacity: Real read FFillOpacity write SetFillOpacity; // 0 to 1
  end;

  { ****************************************************************************
    internal class
  **************************************************************************** }
  TPolygons = class(TPolylines)
  private
    procedure SetItems(I: Integer; const Value: TPolygon);
    function GetItems(I: Integer): TPolygon;
  protected
    function GetOwner: TPersistent; override;
  public
    function Add: TPolygon;
    function Insert(Index: Integer): TPolygon;

    property Items[I: Integer]: TPolygon read GetItems write SetItems; default;
  end;

  { ****************************************************************************
    https://developers.google.com/maps/documentation/javascript/reference?hl=en#Polygon
  **************************************************************************** }
  TGMPolygon = class(TGMPolyline)
  private
    FOnFillOpacityChange: TLinkedComponentChange;
    FOnFillColorChange: TLinkedComponentChange;
  protected
    function GetAPIUrl: string; override;

    function GetItems(I: Integer): TPolygon;

    procedure ShowElements; override;
    function GetCollectionItemClass: TLinkedComponentClass; override;
    function GetCollectionClass: TLinkedComponentsClass; override;
  public
    function Add: TPolygon;

    property Items[I: Integer]: TPolygon read GetItems; default;
  published
    // eventos
    // events
        // from properties
    property OnFillColorChange: TLinkedComponentChange read FOnFillColorChange write FOnFillColorChange;
    property OnFillOpacityChange: TLinkedComponentChange read FOnFillOpacityChange write FOnFillOpacityChange;
  end;

implementation

uses
  SysUtils;

{ TGMPolygon }

function TGMPolygon.Add: TPolygon;
begin
  Result := TPolygon(inherited Add);
end;

function TGMPolygon.GetAPIUrl: string;
begin
  Result := 'https://developers.google.com/maps/documentation/javascript/reference?hl=en#Polygon';
end;

function TGMPolygon.GetCollectionClass: TLinkedComponentsClass;
begin
  Result := TPolygons;
end;

function TGMPolygon.GetCollectionItemClass: TLinkedComponentClass;
begin
  Result := TPolygon;
end;

function TGMPolygon.GetItems(I: Integer): TPolygon;
begin
  Result := TPolygon(inherited Items[i]);
end;

procedure TGMPolygon.ShowElements;
const
  StrParams = '%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s';
var
  Params: string;
  i: Integer;
begin
  if not ExecuteScript('DeleteObjects', IntToStr(IdxList)) then Exit;

  for i:= 0 to VisualObjects.Count - 1 do
  begin
    Params := Format(StrParams, [
                    IntToStr(IdxList),
                    LowerCase(BoolToStr(Items[i].Clickable, True)),
                    LowerCase(BoolToStr(Items[i].Editable, True)),
                    QuotedStr(Map.TColorToStr(Items[i].FillColor)),
                    StringReplace(FloatToStr(Items[i].FillOpacity), ',', '.', [rfReplaceAll]),
                    LowerCase(BoolToStr(Items[i].Geodesic, True)),
                    QuotedStr(Map.TColorToStr(Items[i].StrokeColor)),
                    StringReplace(FloatToStr(Items[i].StrokeOpacity), ',', '.', [rfReplaceAll]),
                    IntToStr(Items[i].StrokeWeight),
                    LowerCase(BoolToStr(Items[i].Visible, True)),
                    QuotedStr(Items[i].PolylineToStr),
                    IntToStr(Items[i].Index),
                    QuotedStr(Items[i].InfoWindow.GetConvertedString),
                    LowerCase(BoolToStr(Items[i].InfoWindow.DisableAutoPan, True)),
                    IntToStr(Items[i].InfoWindow.MaxWidth),
                    IntToStr(Items[i].InfoWindow.PixelOffset.Height),
                    IntToStr(Items[i].InfoWindow.PixelOffset.Width),
                    LowerCase(BoolToStr(Items[i].InfoWindow.CloseOtherBeforeOpen, True))
                    ]);

    ExecuteScript('MakePolygon', Params);
  end;
end;

{ TPolygons }

function TPolygons.Add: TPolygon;
begin
  Result := TPolygon(inherited Add);
end;

function TPolygons.GetItems(I: Integer): TPolygon;
begin
  Result := TPolygon(inherited Items[I]);
end;

function TPolygons.GetOwner: TPersistent;
begin
  Result := TGMPolygon(inherited GetOwner);
end;

function TPolygons.Insert(Index: Integer): TPolygon;
begin
  Result := TPolygon(inherited Insert(Index));
end;

procedure TPolygons.SetItems(I: Integer; const Value: TPolygon);
begin
  inherited SetItem(I, Value);
end;

{ TPolygon }

procedure TPolygon.Assign(Source: TPersistent);
begin
  inherited;

  if Source is TPolyline then
  begin
    FillColor := TPolygon(Source).FillColor;
    FillOpacity := TPolygon(Source).FillOpacity;
  end;
end;

constructor TPolygon.Create(Collection: TCollection);
begin
  inherited;

  FFillOpacity := 0.5;
  FFillColor := clRed;
end;

procedure TPolygon.SetFillColor(const Value: TColor);
begin
  if FFillColor = Value then Exit;

  FFillColor := Value;

  {$IF CompilerVersion < 20}  // ES: si la versin es inferior a D2009 - EN: if the version lower than D2009
  SetColorProperty('PolygonSetFillColor', 'OnFillColorChange', FFillColor);
  {$ELSE}
  SetProperty('PolygonSetFillColor', 'OnFillColorChange', FFillColor);
  {$IFEND}
end;

procedure TPolygon.SetFillOpacity(const Value: Real);
begin
  if FFillOpacity = Value then Exit;

  FFillOpacity := Value;
  if FFillOpacity < 0 then FFillOpacity := 0;
  if FFillOpacity > 1 then FFillOpacity := 1;
  FFillOpacity := Trunc(FFillOpacity * 100) / 100;

  SetProperty('PolygonSetFillOpacity', 'OnFillOpacityChange', FFillOpacity);
end;

end.
